==> fidget-0.0.4b2.tar.gz: fidget-0.0.4b2/fidget/core/code_editor.py <==
"""
a utility code editor module
"""

import re

from fidget.backend.QtCore import Qt, QRect, QSize
from fidget.backend.QtWidgets import QWidget, QPlainTextEdit, QTextEdit
from fidget.backend.QtGui import QColor, QPainter, QTextFormat

from fidget.core.highlighter import Python3Highlighter


class QLineNumberArea(QWidget):
    def __init__(self, editor):
        super().__init__(editor)
        self.codeEditor = editor

    def sizeHint(self):
        return QSize(self.editor.lineNumberAreaWidth(), 0)

    def paintEvent(self, event):
        self.codeEditor.lineNumberAreaPaintEvent(event)


class QCodeEditor(QPlainTextEdit):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.lineNumberArea = QLineNumberArea(self)

        self.blockCountChanged.connect(self.updateLineNumberAreaWidth)
        self.updateRequest.connect(self.updateLineNumberArea)
        self.cursorPositionChanged.connect(self.highlightCurrentLine)
        self.updateLineNumberAreaWidth(0)

    def lineNumberAreaWidth(self):
        digits = 1
        max_value = max(1, self.blockCount())
        while max_value >= 10:
            max_value /= 10
            digits += 1
        space = 3 + self.fontMetrics().width('9') * digits
        return space

    def updateLineNumberAreaWidth(self, _):
        self.setViewportMargins(self.lineNumberAreaWidth(), 0, 0, 0)

    def updateLineNumberArea(self, rect, dy):
        if dy:
            self.lineNumberArea.scroll(0, dy)
        else:
            self.lineNumberArea.update(0, rect.y(), self.lineNumberArea.width(), rect.height())
        if rect.contains(self.viewport().rect()):
            self.updateLineNumberAreaWidth(0)

    def resizeEvent(self, event):
        super().resizeEvent(event)
        cr = self.contentsRect()
        self.lineNumberArea.setGeometry(QRect(cr.left(), cr.top(), self.lineNumberAreaWidth(), cr.height()))

    def highlightCurrentLine(self):
        extraSelections = []
        if not self.isReadOnly():
            selection = QTextEdit.ExtraSelection()
            lineColor = QColor(Qt.yellow).lighter(160)
            selection.format.setBackground(lineColor)
            selection.format.setProperty(QTextFormat.FullWidthSelection, True)
            selection.cursor = self.textCursor()
            selection.cursor.clearSelection()
            extraSelections.append(selection)
        self.setExtraSelections(extraSelections)

    def lineNumberAreaPaintEvent(self, event):
        painter = QPainter(self.lineNumberArea)

        painter.fillRect(event.rect(), Qt.lightGray)

        block = self.firstVisibleBlock()
        blockNumber = block.blockNumber()
        top = self.blockBoundingGeometry(block).translated(self.contentOffset()).top()
        bottom = top + self.blockBoundingRect(block).height()

        # Just to make sure I use the right font
        height = self.fontMetrics().height()
        while block.isValid() and (top <= event.rect().bottom()):
            if block.isVisible() and (bottom >= event.rect().top()):
                number = str(blockNumber + 1)
                painter.setPen(Qt.black)
                painter.drawText(0, top, self.lineNumberArea.width(), height, Qt.AlignRight, number)

            block = block.next()
            top = bottom
            bottom = top + self.blockBoundingRect(block).height()
            blockNumber += 1


class QPyCodeEditor(QCodeEditor):
    INDENT_PATTERN = re.compile(r'\s*')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.highlighter = Python3Highlighter(self.document())

    def keyPressEvent(self, event):
        if (not event.modifiers() and event.key() == Qt.Key_Return) \
                or (event.modifiers() == Qt.KeypadModifier and event.key() == Qt.Key_Enter):
            block = self.textCursor().block()
            text = block.text()
            indent = self.INDENT_PATTERN.match(text)[0]
            ind_style = self.indent_style(indent)
            if text.endswith(':'):
                indent += ind_style
            super().keyPressEvent(event)
            if indent:
                self.insertPlainText(indent)
        elif (not event.modifiers()) and event.key() == Qt.Key_Backspace:
            # we might need to delete 4 spaces at once
            cursor = self.textCursor()
            position_in_block = cursor.positionInBlock()
            text = cursor.block().text()[:position_in_block]
            spaces_to_delete = 0
            while spaces_to_delete < 4:
                pos = len(text) - spaces_to_delete - 1
                if pos < 0 or text[pos] != ' ':
                    break
                spaces_to_delete += 1

            if not spaces_to_delete:
                # nos spaces to delete, let super do its thing
                super().keyPressEvent(event)
            else:
                # we got some spaces behind us, we got this
                for _ in range(spaces_to_delete):
                    self.textCursor().deletePreviousChar()
        else:
            super().keyPressEvent(event)

    @staticmethod
    def indent_style(indent):
        if indent.startswith('\t'):
            return '\t'
        return '    '

==> fidget-0.0.6.tar.gz: fidget-0.0.6/fidget/core/code_editor.py <==
"""
a utility code editor module
"""

import re

from fidget.backend.QtCore import Qt, QRect, QSize
from fidget.backend.QtWidgets import QWidget, QPlainTextEdit, QTextEdit
from fidget.backend.QtGui import QColor, QPainter, QTextFormat, QFontDatabase

from fidget.core.highlighter import Python3Highlighter


class QLineNumberArea(QWidget):
    def __init__(self, editor):
        super().__init__(editor)
        self.codeEditor = editor

    def sizeHint(self):
        return QSize(self.editor.lineNumberAreaWidth(), 0)

    def paintEvent(self, event):
        self.codeEditor.lineNumberAreaPaintEvent(event)


class QCodeEditor(QPlainTextEdit):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.lineNumberArea = QLineNumberArea(self)

        self.blockCountChanged.connect(self.updateLineNumberAreaWidth)
        self.updateRequest.connect(self.updateLineNumberArea)
        self.cursorPositionChanged.connect(self.highlightCurrentLine)
        self.updateLineNumberAreaWidth(0)
        font = QFontDatabase.systemFont(QFontDatabase.FixedFont)
        self.document().setDefaultFont(font)

    def lineNumberAreaWidth(self):
        digits = 1
        max_value = max(1, self.blockCount())
        while max_value >= 10:
            max_value /= 10
            digits += 1
        space = 3 + self.fontMetrics().width('9') * digits
        return space

    def updateLineNumberAreaWidth(self, _):
        self.setViewportMargins(self.lineNumberAreaWidth(), 0, 0, 0)

    def updateLineNumberArea(self, rect, dy):
        if dy:
            self.lineNumberArea.scroll(0, dy)
        else:
            self.lineNumberArea.update(0, rect.y(), self.lineNumberArea.width(), rect.height())
        if rect.contains(self.viewport().rect()):
            self.updateLineNumberAreaWidth(0)

    def resizeEvent(self, event):
        super().resizeEvent(event)
        cr = self.contentsRect()
        self.lineNumberArea.setGeometry(QRect(cr.left(), cr.top(), self.lineNumberAreaWidth(), cr.height()))

    def highlightCurrentLine(self):
        extraSelections = []
        if not self.isReadOnly():
            selection = QTextEdit.ExtraSelection()
            lineColor = QColor(Qt.yellow).lighter(160)
            selection.format.setBackground(lineColor)
            selection.format.setProperty(QTextFormat.FullWidthSelection, True)
            selection.cursor = self.textCursor()
            selection.cursor.clearSelection()
            extraSelections.append(selection)
        self.setExtraSelections(extraSelections)

    def lineNumberAreaPaintEvent(self, event):
        painter = QPainter(self.lineNumberArea)

        painter.fillRect(event.rect(), Qt.lightGray)

        block = self.firstVisibleBlock()
        blockNumber = block.blockNumber()
        top = self.blockBoundingGeometry(block).translated(self.contentOffset()).top()
        bottom = top + self.blockBoundingRect(block).height()

        # Just to make sure I use the right font
        height = self.fontMetrics().height()
        while block.isValid() and (top <= event.rect().bottom()):
            if block.isVisible() and (bottom >= event.rect().top()):
                number = str(blockNumber + 1)
                painter.setPen(Qt.black)
                painter.drawText(0, top, self.lineNumberArea.width(), height, Qt.AlignRight, number)

            block = block.next()
            top = bottom
            bottom = top + self.blockBoundingRect(block).height()
            blockNumber += 1


class QPyCodeEditor(QCodeEditor):
    INDENT_PATTERN = re.compile(r'\s*')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.highlighter = Python3Highlighter(self.document())

    def keyPressEvent(self, event):
        if (not event.modifiers() and event.key() == Qt.Key_Return) \
                or (event.modifiers() == Qt.KeypadModifier and event.key() == Qt.Key_Enter):
            block = self.textCursor().block()
            text = block.text()
            indent = self.INDENT_PATTERN.match(text)[0]
            ind_style = self.indent_style(indent)
            if text.endswith(':'):
                indent += ind_style
            super().keyPressEvent(event)
            if indent:
                self.insertPlainText(indent)
        elif (not event.modifiers()) and event.key() == Qt.Key_Backspace:
            # we might need to delete 4 spaces at once
            cursor = self.textCursor()
            position_in_block = cursor.positionInBlock()
            text = cursor.block().text()[:position_in_block]
            spaces_to_delete = 0
            while spaces_to_delete < 4:
                pos = len(text) - spaces_to_delete - 1
                if pos < 0 or text[pos] != ' ':
                    break
                spaces_to_delete += 1

            if not spaces_to_delete:
                # nos spaces to delete, let super do its thing
                super().keyPressEvent(event)
            else:
                # we got some spaces behind us, we got this
                for _ in range(spaces_to_delete):
                    self.textCursor().deletePreviousChar()
        else:
            super().keyPressEvent(event)

    @staticmethod
    def indent_style(indent):
        if indent.startswith('\t'):
            return '\t'
        return '    '
